From: john stultz <johnstul@us.ibm.com>
To: lkml <linux-kernel@vger.kernel.org>
Cc: Darren Hart <dvhltc@us.ibm.com>,
Nishanth Aravamudan <nacc@us.ibm.com>,
Frank Sorenson <frank@tuxrocks.com>,
Thomas Gleixner <tglx@linutronix.de>,
George Anzinger <george@mvista.com>,
Ulrich Windl <ulrich.windl@rz.uni-regensburg.de>,
Roman Zippel <zippel@linux-m68k.org>
Subject: [RFC][PATCH 1/11] Reduced NTP rework (part 1)
Date: Thu, 29 Sep 2005 17:42:19 -0700 [thread overview]
Message-ID: <1128040939.8195.310.camel@cog.beaverton.ibm.com> (raw)
In-Reply-To: <1128040851.8195.307.camel@cog.beaverton.ibm.com>
All,
With Roman's suggestions, I've been working on reducing the footprint
of my timeofday patches. This is the first of two patches that reworks
some of the interrupt time NTP adjustments so that it could be re-used
with the timeofday patches. The motivation of the change is to logically
separate the code which adjusts xtime and the code that decides, based
on the NTP state variables, how much per tick to adjust xtime.
Thus this patch should not affect the existing behavior, but just
separate the logical functionality so it can be re-used.
This patch applies on top of my ntp-shift-right patch I posted awhile
back.
thanks
-john
timer.c | 88 ++++++++++++++++++++++++++++++++++++++++------------------------
1 files changed, 55 insertions(+), 33 deletions(-)
linux-2.6.14-rc2_timeofday-ntp-part1_B6.patch
============================================
diff --git a/kernel/timer.c b/kernel/timer.c
--- a/kernel/timer.c
+++ b/kernel/timer.c
@@ -620,6 +620,7 @@ static long time_adj; /* tick adjust (
long time_reftime; /* time at last adjustment (s) */
long time_adjust;
long time_next_adjust;
+long time_adjust_step; /* per tick time_adjust step */
/*
* this routine handles the overflow of the microsecond field
@@ -742,46 +743,46 @@ static void second_overflow(void)
#endif
}
-/* in the NTP reference this is called "hardclock()" */
-static void update_wall_time_one_tick(void)
+
+static void ntp_advance(unsigned long interval_ns)
{
- long time_adjust_step, delta_nsec;
+ static unsigned long interval_sum;
- if ( (time_adjust_step = time_adjust) != 0 ) {
- /* We are doing an adjtime thing.
- *
- * Prepare time_adjust_step to be within bounds.
- * Note that a positive time_adjust means we want the clock
- * to run faster.
- *
- * Limit the amount of the step to be in the range
- * -tickadj .. +tickadj
- */
- time_adjust_step = min(time_adjust_step, (long)tickadj);
- time_adjust_step = max(time_adjust_step, (long)-tickadj);
+ /* increment the interval sum */
+ interval_sum += interval_ns;
- /* Reduce by this step the amount of time left */
- time_adjust -= time_adjust_step;
- }
- delta_nsec = tick_nsec + time_adjust_step * 1000;
- /*
- * Advance the phase, once it gets to one microsecond, then
- * advance the tick more.
- */
- time_phase += time_adj;
- if ((time_phase >= FINENSEC) || (time_phase <= -FINENSEC)) {
- long ltemp = shift_right(time_phase, (SHIFT_SCALE - 10));
- time_phase -= ltemp << (SHIFT_SCALE - 10);
- delta_nsec += ltemp;
+ /* calculate the per tick singleshot adjtime adjustment step */
+ while (interval_ns >= tick_nsec) {
+ time_adjust_step = time_adjust;
+ if (time_adjust_step) {
+ /* We are doing an adjtime thing.
+ *
+ * Prepare time_adjust_step to be within bounds.
+ * Note that a positive time_adjust means we want the clock
+ * to run faster.
+ *
+ * Limit the amount of the step to be in the range
+ * -tickadj .. +tickadj
+ */
+ time_adjust_step = min(time_adjust_step, (long)tickadj);
+ time_adjust_step = max(time_adjust_step, (long)-tickadj);
+
+ /* Reduce by this step the amount of time left */
+ time_adjust -= time_adjust_step;
+ }
+ interval_ns -= tick_nsec;
}
- xtime.tv_nsec += delta_nsec;
- time_interpolator_update(delta_nsec);
/* Changes by adjtime() do not take effect till next tick. */
- if (time_next_adjust != 0) {
+ if (time_next_adjust) {
time_adjust = time_next_adjust;
time_next_adjust = 0;
}
+
+ while (interval_sum > NSEC_PER_SEC) {
+ interval_sum -= NSEC_PER_SEC;
+ second_overflow();
+ }
}
/*
@@ -793,14 +794,35 @@ static void update_wall_time_one_tick(vo
*/
static void update_wall_time(unsigned long ticks)
{
+ long delta_nsec;
+
do {
ticks--;
- update_wall_time_one_tick();
+
+ /* Calculate the nsec delta using the
+ * precomputed NTP adjustments:
+ * tick_nsec, time_adjust_step, time_adj
+ */
+ delta_nsec = tick_nsec + time_adjust_step * 1000;
+ /*
+ * Advance the phase, once it gets to one microsecond, then
+ * advance the tick more.
+ */
+ time_phase += time_adj;
+ if ((time_phase >= FINENSEC) || (time_phase <= -FINENSEC)) {
+ long ltemp = shift_right(time_phase, (SHIFT_SCALE - 10));
+ time_phase -= ltemp << (SHIFT_SCALE - 10);
+ delta_nsec += ltemp;
+ }
+
+ xtime.tv_nsec += delta_nsec;
if (xtime.tv_nsec >= 1000000000) {
xtime.tv_nsec -= 1000000000;
xtime.tv_sec++;
- second_overflow();
}
+ ntp_advance(tick_nsec);
+ time_interpolator_update(delta_nsec);
+
} while (ticks);
}
next parent reply other threads:[~2005-09-30 0:42 UTC|newest]
Thread overview: 12+ messages / expand[flat|nested] mbox.gz Atom feed top
[not found] <1128040851.8195.307.camel@cog.beaverton.ibm.com>
2005-09-30 0:42 ` john stultz [this message]
2005-09-30 0:44 ` [RFC][PATCH 2/11] Reduced NTP rework (part 2) john stultz
2005-09-30 0:45 ` [RFC][PATCH 3/11] timesource management code john stultz
2005-09-30 0:46 ` [PATCH 4/11] generic timeofday core subsystem john stultz
2005-09-30 0:48 ` [RFC][PATCH 5/11] generic timeofday i386 arch specific changes, part 1 john stultz
2005-09-30 0:49 ` [RFC][PATCH 6/11] generic timeofday i386 arch specific changes, part 2 john stultz
2005-09-30 0:49 ` [RFC][PATCH 7/11] generic timeofday i386 arch specific changes, part 3 john stultz
2005-09-30 0:50 ` [RFC][PATCH 8/11] generic timeofday i386 arch specific changes, part 4 john stultz
2005-09-30 0:51 ` [RFC][PATCH 9/11] generic timeofday i386 arch specific changes, part 5 john stultz
2005-09-30 0:52 ` [RFC][PATCH 10/11] generic timeofday i386 arch specific changes, part 6 john stultz
2005-09-30 0:53 ` [RFC][PATCH 11/11] generic timeofday i386 specific timesources john stultz
2005-09-30 1:02 ` [RFC][PATCH] Generic Timekeeping Rework (cumulative - bz'ed) (v. B6) john stultz
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=1128040939.8195.310.camel@cog.beaverton.ibm.com \
--to=johnstul@us.ibm.com \
--cc=dvhltc@us.ibm.com \
--cc=frank@tuxrocks.com \
--cc=george@mvista.com \
--cc=linux-kernel@vger.kernel.org \
--cc=nacc@us.ibm.com \
--cc=tglx@linutronix.de \
--cc=ulrich.windl@rz.uni-regensburg.de \
--cc=zippel@linux-m68k.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®